using BidirectionalShare;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Tcp;
using System.Runtime.Serialization.Formatters;
using System.Text;

namespace JrmServer
{
	class JrmServer
	{
		/// <summary>
		/// Main application starts here
		/// </summary>
		/// <param name="args">Maybe pass in the port number</param>
		static void Main( string[] args )
		{
			// Creating a custom formatter for a TcpChannel sink chain.
			BinaryServerFormatterSinkProvider provider = new BinaryServerFormatterSinkProvider();
			provider.TypeFilterLevel = TypeFilterLevel.Full;

			// Creating the IDictionary to set the port on the channel instance.
			IDictionary props = new Hashtable();
			props["port"] = 8392;					// This must match number on client

			// Pass the properties for the port setting and the server provider in the server chain argument. (Client remains null here.)
			TcpChannel m_tcpChannel = new TcpChannel(props, null, provider);
			ChannelServices.RegisterChannel( m_tcpChannel, false );

			// Create the server object for clients to connect to
			RemotingConfiguration.RegisterWellKnownServiceType(
				typeof( ClientComms ),
				"RemoteServer",
				WellKnownObjectMode.Singleton );

			// Loop here until the uers is done
			Console.WriteLine( "Type some text to send to the client, or just press enter to exit" );
			string s;
			while( ( s = Console.ReadLine() ) != "" )
				ClientComms.FireNewBroadcastedMessageEvent( s );

			Console.WriteLine( "Sooo long and thanks for all the fish." );
		}
	}

	/// <summary>
	/// An instance of this class is exposed to the client allowing it to call into the server here
	/// </summary>
	public class ClientComms : MarshalByRefObject, ICallsToServer
	{
		/// <summary>
		/// Function to call the server from the client. This is not in the GUI thread.
		/// </summary>
		/// <param name="n">Some number</param>
		/// <returns>Some interesting text</returns>
		public string SomeSimpleFunction( int n )
		{
			Console.Write( "  Client sent : {0}      \r", n );
			return "Server says : " + n.ToString();
		}

		/// <summary>
		/// Local copy of event holding a collection
		/// </summary>
		private static event NotifyCallback s_notify;

		/// <summary>
		/// Add or remove callback destinations on the client
		/// </summary>
		public event NotifyCallback Notify
		{
			add	   { s_notify = value; }
			remove { Console.WriteLine( "TODO : Notify remove." ); }
		}

		/// <summary>
		/// Call this method to send the string to the client. This call will throw an exception
		/// if the client has gone or network is down
		/// </summary>
		/// <param name="s">Some test to send to client</param>
		public static void FireNewBroadcastedMessageEvent( string s )
		{
			Console.WriteLine( "Broadcasting... Sending : {0}", s );
			s_notify( s );
		}
	}
}

